"use strict";
/*
 * Copyright (c) 2019, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const admzip = require("adm-zip");
const constants_1 = require("../constants");
const types_1 = require("../types");
command_1.core.Messages.importMessagesDirectory(__dirname);
const messages = command_1.core.Messages.loadMessages('@salesforce/sfdx-diff', 'diff');
class MetadataApiService {
    // tslint:disable-next-line:member-ordering
    static getInstance() {
        if (!MetadataApiService.instance) {
            MetadataApiService.instance = new MetadataApiService();
        }
        return MetadataApiService.instance;
    }
    initialize(connection) {
        this.connection = connection;
    }
    buildRetrieveRequest(typeName, memberName) {
        return {
            apiVersion: constants_1.API_VERSION,
            unpackaged: {
                types: { name: typeName, members: memberName },
                version: constants_1.API_VERSION_STRING
            },
            singlePackage: false
        };
    }
    async metadataRetrieveID(request) {
        let retrieveID = '';
        await this.connection.metadata.retrieve(request, (err, res) => {
            if (err) {
                throw new Error(err.message);
            }
            retrieveID = res.id;
        });
        return retrieveID;
    }
    /**
     * This allows us to wrap async functions, like metadata retrieve, so we can
     * execute them as Immediately Invoked Function Expression in order to poll
     * for its results
     * @param retrieveID metadata retrieve Id
     * @param timeout period of time in milliseconds before finish polling
     * @param interval time interval between polls in milliseconds
     */
    async metadataRetrievePoll(retrieveID, timeout, interval) {
        timeout = timeout || 2000;
        interval = interval || 100;
        const endTime = Number(new Date()) + timeout;
        const checkRetrieve = async (resolve, reject) => {
            const result = this.connection.metadata.checkRetrieveStatus(retrieveID);
            if (result) {
                let status = '';
                await result.then(res => {
                    // @ts-ignore
                    status = res.status;
                });
                switch (status) {
                    case types_1.RetrieveStatusEnum.Succeeded:
                        resolve(result);
                        break;
                    case types_1.RetrieveStatusEnum.InProgress:
                    case types_1.RetrieveStatusEnum.Pending:
                    case '':
                        if (Number(new Date()) < endTime) {
                            setTimeout(checkRetrieve, interval, resolve, reject);
                        }
                        else {
                            reject(new Error(messages.getMessage('md_request_timeout')));
                        }
                        break;
                    case types_1.RetrieveStatusEnum.Failed:
                        reject(new Error(messages.getMessage('md_request_fail')));
                        break;
                    default:
                        if (Number(new Date()) < endTime) {
                            setTimeout(checkRetrieve, interval, resolve, reject);
                        }
                        else {
                            reject(new Error(messages.getMessage('md_request_timeout')));
                        }
                }
            }
            else if (Number(new Date()) < endTime) {
                setTimeout(checkRetrieve, interval, resolve, reject);
            }
            else {
                reject(new Error(messages.getMessage('md_request_timeout')));
            }
        };
        return new Promise(checkRetrieve);
    }
    getFileFromZipFile(zipFileString, fileName) {
        const decodedZip = Buffer.from(zipFileString, 'base64');
        const zip = new admzip(decodedZip);
        const zipEntries = zip.getEntries();
        const fileNameRegex = new RegExp(fileName, 'i');
        let textFile = '';
        zipEntries.forEach(entry => {
            if (entry.entryName.match(fileNameRegex)) {
                textFile = zip.readAsText(entry);
            }
        });
        return textFile;
    }
}
exports.MetadataApiService = MetadataApiService;
//# sourceMappingURL=metadataApiService.js.map